/*
 *  soapUI, copyright (C) 2004-2009 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.support.components;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.support.Tools;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.xml.XmlUtils;
import com.teamdev.jxbrowser.WebBrowser;
import com.teamdev.jxbrowser.WebBrowserFactory;
import com.teamdev.jxbrowser.WebBrowserWindow;
import com.teamdev.jxbrowser.WindowCreator;
import com.teamdev.jxbrowser.event.LocationEvent;
import com.teamdev.jxbrowser.event.RequestAdapter;
import com.teamdev.jxbrowser.event.StateEvent;
import com.teamdev.jxbrowser.event.StatusChangeEvent;
import com.teamdev.jxbrowser.event.StatusChangeListener;
import com.teamdev.xpcom.Xpcom;

public class BrowserComponent
{
	private WebBrowser browser;
	private static WebBrowserFactory webBrowserFactory;
	private JPanel panel = new JPanel( new BorderLayout() );
	private JPanel statusBar;
	private JLabel statusLabel;
	private WebBrowserWindow browserWindowAdapter = new BrowserWindowAdapter();
	private final boolean addToolbar;

	private static Boolean initialized = false;

	public BrowserComponent( boolean addToolbar )
	{
		if( !initialized )
		{
			synchronized( initialized )
			{
				if( !initialized )
				{
					try
					{
						Xpcom.initialize( Xpcom.AWT );
						initialized = true;
					}
					catch( Throwable t )
					{
						t.printStackTrace();
					}
				}
			}
		}

		this.addToolbar = addToolbar;
	}

	public Component getComponent()
	{
		if( browser == null )
		{
			initBrowser();
			navigate( "about:blank", null );

			statusBar = new JPanel();
			statusLabel = new JLabel();
			statusBar.add( statusLabel, BorderLayout.CENTER );

			if( addToolbar )
				panel.add( buildToolbar(), BorderLayout.NORTH );

			panel.add( browser.getComponent(), BorderLayout.CENTER );
			panel.add( statusBar, BorderLayout.SOUTH );
		}

		return panel;
	}

	private Component buildToolbar()
	{
		JXToolBar toolbar = UISupport.createToolbar();

		toolbar.addFixed( UISupport.createToolbarButton( new BackAction() ) );
		toolbar.addRelatedGap();
		toolbar.addFixed( UISupport.createToolbarButton( new ForwardAction() ) );

		toolbar.addGlue();

		return toolbar;
	}

	private class BackAction extends AbstractAction
	{
		public BackAction()
		{
			putValue( SMALL_ICON, UISupport.createImageIcon( "/arrow_left.png" ) );
			putValue( Action.SHORT_DESCRIPTION, "Navigate to previous selection" );
		}

		public void actionPerformed( ActionEvent e )
		{
			if( browser.getHistory().getCurrentPosition() == 0 )
				Toolkit.getDefaultToolkit().beep();
			else
				browser.goBack();
		}
	}

	private class ForwardAction extends AbstractAction
	{
		public ForwardAction()
		{
			putValue( SMALL_ICON, UISupport.createImageIcon( "/arrow_right.png" ) );
			putValue( Action.SHORT_DESCRIPTION, "Navigate to next selection" );
		}

		public void actionPerformed( ActionEvent e )
		{
			// WebBrowserHistory history = browser.getHistory();
			// List entries = history.getEntries();
			// if( history.getCurrentPosition() == entries.size()-1 )
			// Toolkit.getDefaultToolkit().beep();
			// else
			browser.goForward();
		}
	}

	private void initBrowser()
	{
		if( webBrowserFactory == null )
			webBrowserFactory = WebBrowserFactory.getInstance();

		browser = webBrowserFactory.createBrowser();
		browser.addStatusChangeListener( new StatusChangeListener()
		{
			public void statusChanged( final StatusChangeEvent event )
			{
				if( statusLabel != null )
				{
					SwingUtilities.invokeLater( new Runnable()
					{
						public void run()
						{
							statusLabel.setText( event.getStatus() );
						}
					} );
				}
			}
		} );

		browser.addRequestListener( new RequestAdapter()
		{
			public void stateChanged( final StateEvent event )
			{
				// System.out.println("Got event for url:" +
				// event.getRequestUrlName());
			}
		} );

		browser.setWindowCreator( new WindowCreator()
		{
			public WebBrowserWindow createChildWindow( Component parentComponent, long flags )
			{
				return browserWindowAdapter;
			}
		} );

		browser.deactivate();
	}

	public void release()
	{
		if( browser != null )
		{
			browser.stop();
			browser.deactivate();
			browser.dispose();
		}
		browser = null;
	}

	public void setContent( String contentAsString, String contentType, String contextUri )
	{
		if( browser == null )
		{
			initBrowser();
		}
		browser.setContentWithContext( contentAsString, contentType, contextUri );
	}

	public void setContent( String content, String contentType )
	{
		// setContent( content, contentType, null );

		if( browser == null )
		{
			initBrowser();
		}

		try
		{
			browser.setContent( content, contentType );
		}
		catch( Throwable t )
		{
			SoapUI.logError( t );
		}
	}

	public boolean navigate( String url, String errorPage )
	{
		if( browser == null )
		{
			initBrowser();
		}

		SwingUtilities.invokeLater( new Navigator( url ) );

		return true;
	}

	public String getContent()
	{
		return browser == null ? null : XmlUtils.serialize( browser.getDocument() );
	}

	private final class Navigator implements Runnable
	{
		private final String url;

		private Navigator( String url )
		{
			this.url = url;
		}

		public void run()
		{
			browser.navigate( url );
		}
	}

	private class BrowserWindowAdapter implements WebBrowserWindow
	{
		public void close()
		{
		}

		public boolean isClosed()
		{
			return true;
		}

		public void setModal( boolean arg0 )
		{
		}

		public void setSize( int arg0, int arg1 )
		{
		}

		public void setVisible( boolean arg0 )
		{
		}

		public void setWebBrowser( WebBrowser arg0 )
		{
			if( arg0 != null )
			{
				arg0.addRequestListener( new RequestAdapter()
				{

					@Override
					public void locationChanged( final LocationEvent arg0 )
					{
						if( !arg0.getLocation().equals( "about:blank" ) )
						{
							if( UISupport.confirm( "Open url [" + arg0.getLocation() + "] in external browser?", "Open URL" ) )
								Tools.openURL( arg0.getLocation() );

							arg0.getWebBrowser().stop();
							arg0.getWebBrowser().deactivate();
							arg0.getWebBrowser().dispose();
							arg0.getWebBrowser().removeRequestListener( this );
						}
						// Toolkit.getDefaultToolkit().beep();
					}
				} );

				// arg0.stop();
				// arg0.deactivate();
				// arg0.dispose();
				//
				// Toolkit.getDefaultToolkit().beep();
			}

		}
	}

}
